/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.engvew.engstelle;

import static org.junit.Assert.assertTrue;

import java.util.LinkedList;
import java.util.List;

import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.vew.engvew.testtools.InputData;
import de.bsvrz.vew.engvew.testtools.JUnitHelper;

public class TestUnfall 
{

  private static SystemObject _sysObj = null;
  private static List<InputData> _testData = new LinkedList<InputData>();
  
    
  @BeforeClass
  public static void setUpBeforeClass() throws Exception 
  {
    _sysObj = JUnitHelper.getInstance().erzeugeUnfall();
    
    DataModel dm = JUnitHelper.getInstance().getConnection().getDataModel();
    Aspect asp = dm.getAspect("asp.parameterSoll");
    AttributeGroup atg = dm.getAttributeGroup("atg.situationsEigenschaften"); 
    DataDescription ddZeit =  new DataDescription(atg, asp);
    atg = dm.getAttributeGroup("atg.engstellenVerwaltungParameter");
    DataDescription ddParam =  new DataDescription(atg, asp);
  
    long aktuelleZeit = JUnitHelper.getInstance().getConnection().getTime();
    ResultData zeitlicheGueltigkeit, parameter; 
    
    /* 1. Testfall */
    zeitlicheGueltigkeit = new ResultData(_sysObj, ddZeit, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit - 50 * 60 * 1000, // Unfall begann vor 50min. 
                                                                 35 * 60 * 1000));              // Unfallende 35min. nach Beginn
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataWartezeitNachUnfall(20 * 60 * 1000)); // Wartezeit nach Rumung Unfall ist 20min.
    _testData.add(new InputData(parameter, zeitlicheGueltigkeit, null, null, null));
    
    /* 2. Testfall */
    zeitlicheGueltigkeit = new ResultData(_sysObj, ddZeit, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit - 60 * 60 * 1000, // Unfall begann vor 60min. 
                                                                 35 * 60 * 1000));              // Unfallende 35min. nach Beginn
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataWartezeitNachUnfall(20 * 60 * 1000)); // Wartezeit nach Rumung Unfall ist 20min.
    _testData.add(new InputData(parameter, zeitlicheGueltigkeit, null, null, null));

    /* 3. Testfall */
    zeitlicheGueltigkeit = new ResultData(_sysObj, ddZeit, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit - 50 * 60 * 1000, // Unfall begann vor 50min. 
                                                                 35 * 60 * 1000));              // Unfallende 35min. nach Beginn
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataWartezeitNachUnfall(10 * 60 * 1000)); // Wartezeit nach Rumung Unfall ist 10min.
    _testData.add(new InputData(parameter, zeitlicheGueltigkeit, null, null, null));
    
    /* 4. Testfall */
    zeitlicheGueltigkeit = new ResultData(_sysObj, ddZeit, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit - 60 * 60 * 1000, // Unfall begann vor 60min. 
                                                                 35 * 60 * 1000));              // Unfallende 35min. nach Beginn
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataWartezeitNachUnfall(30 * 60 * 1000)); // Wartezeit nach Rumung Unfall ist 30min.
    _testData.add(new InputData(parameter, zeitlicheGueltigkeit, null, null, null));
        
    /* 5. Testfall */
    zeitlicheGueltigkeit = new ResultData(_sysObj, ddZeit, aktuelleZeit, 
        JUnitHelper.getInstance().getDataSituationsEigenschaften(aktuelleZeit + 10 * 60 * 1000, // Unfall begint in Zukunft (10 min.) 
                                                                 35 * 60 * 1000));              // Unfallende 35min. nach Beginn
    parameter = new ResultData(JUnitHelper.getInstance().getEngVew().getObjekt(), ddParam, aktuelleZeit,
        JUnitHelper.getInstance().getDataWartezeitNachUnfall(20 * 60 * 1000)); // Wartezeit nach Rumung Unfall ist 20min.
    _testData.add(new InputData(parameter, zeitlicheGueltigkeit, null, null, null));
    
    JUnitHelper.getInstance().getConnection().sleep(JUnitHelper.DELAY_INIT);
  }

  @AfterClass
  public static void tearDownAfterClass() throws Exception 
  {
    JUnitHelper.getInstance().loescheUnfall(_sysObj);
    _sysObj = null;
    _testData.clear();
  }
  
  @Before
  public void setUp() throws Exception 
  {
    
  }

  @After
  public void tearDown() throws Exception 
  {
   
  }

  @Test
  public final void testPruefeAblauf() 
  {
    Unfall unfall = null;
    
    for (Situation lauf: JUnitHelper.getInstance().getEngVew().getVerwalteteObjekte())
    {
      if (lauf.getObjekt().equals(_sysObj))
      {
        unfall = (Unfall)lauf;
        break;
      }
    }
    assertTrue("Unfall wird verwaltet", unfall != null);
    int laufendeTestNr = 0;
    
    for (InputData inputData: _testData)
    {
      unfall.update(new ResultData[] { inputData.situationsEigenschaften });
      JUnitHelper.getInstance().getEngVew().update(new ResultData[] { inputData.engstellenVerwaltung });
      
      JUnitHelper.getInstance().getConnection().sleep(JUnitHelper.DELAY_UPDATE);
      
      laufendeTestNr += 1;
      
      long result = unfall.pruefeAblauf();
      
      if (laufendeTestNr == 1)
      {
        assertTrue("Unfallprfung ist nicht verletzt", result == 0);
      }
      if (laufendeTestNr == 2)
      {
        assertTrue("Unfallprfung ist verletzt - Unfall ist seit 25min. abgelaufen", result == 25);
      }
      if (laufendeTestNr == 3)
      {
        assertTrue("Unfallprfung ist nicht verletzt - Unfall ist seit 15min. abgelaufen", result == 15);
      }
      if (laufendeTestNr == 4)
      {
        assertTrue("Unfallprfung ist nicht verletzt", result == 0);
      }
      if (laufendeTestNr == 5)
      {
        assertTrue("Unfallprfung ist nicht verletzt", result == 0);
      }
      System.out.println("Unfallprfung Nr. " + laufendeTestNr + ": OK");
    }
  }

}
