/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Dambach-Werke GmbH
 * Elektronische Leitsysteme
 * Fritz-Minhardt-Str. 1
 * 76456 Kuppenheim
 * Phone: +49-7222-402-0
 * Fax: +49-7222-402-200
 * mailto: info@els.dambach.de
 */

package de.bsvrz.vew.engvew.testtools;

import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.Data.Array;
import de.bsvrz.dav.daf.main.Data.ReferenceArray;
import de.bsvrz.dav.daf.main.Data.TextArray;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.ConfigurationChangeException;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.DynamicObject;
import de.bsvrz.dav.daf.main.config.DynamicObjectType;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.dambach.konfiguration.netz.Konfiguration;
import de.bsvrz.sys.funclib.operatingMessage.MessageSender;
import de.bsvrz.vew.engvew.verwaltung.VerwaltungEngstellen;

public class JUnitHelper extends DavConnectionFromProperties
{
  public final static long DELAY_INIT = 1000;
  public final static long DELAY_UPDATE = 1500;
  
  public final static String NETZ_PID = "verkehrsModellNetz.testBW";
  
  public final static String UNFALL_PID_PREFIX = "situation.Unfall";
  public final static String UNFALL_NAME_PREFIX = "Unfall";
  public final static String UNFALL_MENGE = "Unflle";
  
  public final static String BAUSTELLE_PID_PREFIX = "situation.Baustelle";
  public final static String BAUSTELLE_NAME_PREFIX = "Baustelle";
  public final static String BAUSTELLEN_MENGE = "Baustellen";
  
  public final static String STAU_PID_PREFIX = "situation.Stau";
  public final static String STAU_NAME_PREFIX = "Stau";
  public final static String STAU_MENGE = "Staus";
  
  private JUnitHelper()
  {
    super();
    
    MessageSender.getInstance().setApplicationLabel(
        getConnection().getLocalApplicationObject().getName());
    
    // Initialisiere Objekte des/der Netzes/Netze
    Konfiguration.getInstanz().bestimmeObjekte(getConnection(), NETZ_PID, "");

    _netz = (ConfigurationObject) getConnection().getDataModel().getObject(NETZ_PID);
    _konfigObjekt = getConnection().getLocalConfigurationAuthority();
    
    // Initialisiere Verwaltung(en) der Engstellen
    _engvew = new VerwaltungEngstellen(getConnection(), _netz, _konfigObjekt);
  }
  
  private static JUnitHelper _instance = null;
  
  public synchronized static JUnitHelper getInstance()
  {
    if (_instance == null)
    {
      _instance = new JUnitHelper();
    }
    return _instance;
  }
    
  private ConfigurationObject _netz; 
  private ConfigurationObject _konfigObjekt;
  
  private VerwaltungEngstellen _engvew;
  
  public VerwaltungEngstellen getEngVew()
  {
    return _engvew;
  }
  
  private ConfigurationArea getDefaultConfigurationArea()
  {
    ConfigurationArea ca = null;
    DataModel dm = getConnection().getDataModel();
    ConfigurationObject co = dm.getConfigurationAuthority();
    AttributeGroup atg = dm.getAttributeGroup("atg.konfigurationsVerantwortlicherEigenschaften");
    Data data = co.getConfigurationData(atg);
    if (data != null)
    {
      TextArray array = data.getTextArray("defaultBereich");
      if (array.getLength() > 0)
      {
        ca = (ConfigurationArea)dm.getObject(array.getText(0));
      }
    }
    if (ca == null)
    {
      throw new IllegalArgumentException("Es konnte kein Default-Konfigurationsbereich bestimmt werden");
    }
    return ca;
  }
  
  public DynamicObject erzeugeObjekt(String typPid, MutableSet menge, String pidPrefix, String namePrefix)
  {
    DynamicObjectType typ = (DynamicObjectType)getConnection().getDataModel().getType(typPid);
    DynamicObject dynObj = null;
    short ext = 1;
    
    while(ext < Short.MAX_VALUE && getConnection().getDataModel().getObject(pidPrefix + ext) != null)
    {
      ext += 1;
    }
    
    try
    {
      dynObj = getDefaultConfigurationArea().createDynamicObject(typ, pidPrefix + ext, namePrefix + ext, null);
      menge.add(dynObj);
    }
    catch(ConfigurationChangeException e)
    {
      if (dynObj != null)
      {
        try
        {
          dynObj.invalidate();
        }
        catch(ConfigurationChangeException ex)
        {
          // man kann nichts mehr tun
        }
      }
      throw new RuntimeException("Fehler bei Erzeugen eines dynamisches Objekts vom Typ " + typ.getPid(), e);
    }
    
    return dynObj;
  }
  
  public DynamicObject erzeugeUnfall()
  {
    return erzeugeObjekt("typ.unfall", _netz.getMutableSet(UNFALL_MENGE), UNFALL_PID_PREFIX, UNFALL_NAME_PREFIX);
  }
  
  public DynamicObject erzeugeBaustelle()
  {
    return erzeugeObjekt("typ.baustelle", _netz.getMutableSet(BAUSTELLEN_MENGE), BAUSTELLE_PID_PREFIX, BAUSTELLE_NAME_PREFIX);
  }
  
  public DynamicObject erzeugeStau()
  {
    return erzeugeObjekt("typ.stau", _netz.getMutableSet(STAU_MENGE), STAU_PID_PREFIX, STAU_NAME_PREFIX);
  }
  
  private void loescheObjekt(SystemObject objekt, String typ, MutableSet menge)
  {
    if (!objekt.isOfType(typ))
    {
      throw new IllegalArgumentException("Objekt " + objekt + " ist nicht vom Typ " + typ);
    }
    
    try
    {
      menge.remove(objekt);
      ((DynamicObject)objekt).invalidate();
    }
    catch (ConfigurationChangeException e)
    {
      throw new RuntimeException("Fehler beim Lschen von Objekt " + objekt, e);
    }
  }
  
  public void loescheUnfall(SystemObject unfall)
  {
    loescheObjekt(unfall, "typ.unfall", _netz.getMutableSet(UNFALL_MENGE));
  }
  
  public void loescheBaustelle(SystemObject baustelle)
  {
    loescheObjekt(baustelle, "typ.baustelle", _netz.getMutableSet(BAUSTELLEN_MENGE));
  }
  
  public void loescheStau(SystemObject stau)
  {
    loescheObjekt(stau, "typ.stau", _netz.getMutableSet(STAU_MENGE));
  }
  
  public Data getDataSituationsEigenschaften(long startZeit, long dauer, 
      SystemObject[] strassenSegmente, long startOffset, long endOffset)
  {
    DataModel dm = JUnitHelper.getInstance().getConnection().getDataModel();
    AttributeGroup atg = dm.getAttributeGroup("atg.situationsEigenschaften");
        
    Data data = JUnitHelper.getInstance().getConnection().createData(atg);
    data.getItem("StartZeit").asTimeValue().setMillis(startZeit);
    data.getItem("Dauer").asTimeValue().setMillis(dauer);
    
    if (strassenSegmente != null)
    {
      ReferenceArray array = data.getItem("StraenSegment").asReferenceArray();
      array.setLength(strassenSegmente.length);
      for (int i = 0; i < strassenSegmente.length; i += 1)
      {
        array.getReferenceValue(i).setSystemObject(strassenSegmente[i]);
      }
      data.getItem("StartOffset").asUnscaledValue().set(startOffset);
      data.getItem("EndOffset").asUnscaledValue().set(endOffset);
      
    }
    else
    {
      data.getItem("StraenSegment").asArray().setLength(0);
      data.getItem("StartOffset").asUnscaledValue().set(-1);
      data.getItem("EndOffset").asUnscaledValue().set(-1);
    }
    
    return data;
  }
  
  public Data getDataSituationsEigenschaften(long startZeit, long dauer)
  {
    return getDataSituationsEigenschaften(startZeit, dauer, null, -1, -1);
  }

  public Data getDataBaustellenEigenschaften(String status, int kapazitaet)
  {
    DataModel dm = JUnitHelper.getInstance().getConnection().getDataModel();
    AttributeGroup atg = dm.getAttributeGroup("atg.baustellenEigenschaften");
    
    Data data = JUnitHelper.getInstance().getConnection().createData(atg);
    data.getItem("Status").asTextValue().setText(status);
    data.getItem("RestKapazitt").asUnscaledValue().set(kapazitaet);
    
    return data;
  }
  
  public Data getDataBaustellenEigenschaftenErweitert(int anzahl, String[] lage)
  {
    DataModel dm = JUnitHelper.getInstance().getConnection().getDataModel();
    AttributeGroup atg = dm.getAttributeGroup("atg.baustellenEigenschaftenErweitert");
    
    Data data = JUnitHelper.getInstance().getConnection().createData(atg);
    data.getItem("AnzahlGesperrteFahrstreifen").asUnscaledValue().set(anzahl);
    if (lage != null && lage.length > 0)
    {
      if (anzahl != lage.length)
      {
        throw new IllegalArgumentException("Anzahl gesperrter FS ist nicht konsistent mit der Lage gesperrter FS");
      }
      Array array = data.getItem("LageGesperrteFahrstreifen").asArray(); 
      array.setLength(lage.length);
      for (int i = 0; i < lage.length; i += 1)
      {
        array.getItem(i).asTextValue().setText(lage[i]);
      }
    }
    else
    {
      if (anzahl != 0)
      {
        throw new IllegalArgumentException("Anzahl gesperrter FS ist nicht konsistent mit der Lage gesperrter FS");
      }
      data.getItem("LageGesperrteFahrstreifen").asArray().setLength(0);
    }
    
    return data;
  }
  
  public Data getDataWartezeitNachUnfall(long warteZeitNachUnfall)
  {
    DataModel dm = JUnitHelper.getInstance().getConnection().getDataModel();
    AttributeGroup atg = dm.getAttributeGroup("atg.engstellenVerwaltungParameter");
        
    Data data = JUnitHelper.getInstance().getConnection().createData(atg);
    data.getItem("MaxVerkehrsStrkeGesperrteFahrStreifen").asUnscaledValue().set(-1);
    data.getItem("FaktorMaxVerkehrsStrkeStromabwrts").asScaledValue().set(0.0);
    data.getItem("MaxLngeStauberBaustellenEnde").asUnscaledValue().set(-1);
    data.getItem("WartezeitMeldungNachRumungUnfall").asTimeValue().setMillis(warteZeitNachUnfall);
        
    return data;
  }

  public Data getDataMaxLaengeBaustellenEnde(long laengeBaustellenEnde)
  {
    DataModel dm = JUnitHelper.getInstance().getConnection().getDataModel();
    AttributeGroup atg = dm.getAttributeGroup("atg.engstellenVerwaltungParameter");
        
    Data data = JUnitHelper.getInstance().getConnection().createData(atg);
    data.getItem("MaxVerkehrsStrkeGesperrteFahrStreifen").asUnscaledValue().set(-1);
    data.getItem("FaktorMaxVerkehrsStrkeStromabwrts").asScaledValue().set(0.0);
    data.getItem("MaxLngeStauberBaustellenEnde").asUnscaledValue().set(laengeBaustellenEnde);
    data.getItem("WartezeitMeldungNachRumungUnfall").asTimeValue().setMillis(0);
        
    return data;
  }
  
  public Data getDataFaktorMaxVerkehrsStaerke(float faktorMaxVerkehrsStaerke)
  {
    DataModel dm = JUnitHelper.getInstance().getConnection().getDataModel();
    AttributeGroup atg = dm.getAttributeGroup("atg.engstellenVerwaltungParameter");
        
    Data data = JUnitHelper.getInstance().getConnection().createData(atg);
    data.getItem("MaxVerkehrsStrkeGesperrteFahrStreifen").asUnscaledValue().set(-1);
    data.getItem("FaktorMaxVerkehrsStrkeStromabwrts").asScaledValue().set(faktorMaxVerkehrsStaerke);
    data.getItem("MaxLngeStauberBaustellenEnde").asUnscaledValue().set(-1);
    data.getItem("WartezeitMeldungNachRumungUnfall").asTimeValue().setMillis(0);
        
    return data;
  }
  
  public Data getDataMaxVsGesperrteFs(long maxVsGesperrteFs)
  {
    DataModel dm = JUnitHelper.getInstance().getConnection().getDataModel();
    AttributeGroup atg = dm.getAttributeGroup("atg.engstellenVerwaltungParameter");
        
    Data data = JUnitHelper.getInstance().getConnection().createData(atg);
    data.getItem("MaxVerkehrsStrkeGesperrteFahrStreifen").asUnscaledValue().set(maxVsGesperrteFs);
    data.getItem("FaktorMaxVerkehrsStrkeStromabwrts").asScaledValue().set(0.0);
    data.getItem("MaxLngeStauberBaustellenEnde").asUnscaledValue().set(-1);
    data.getItem("WartezeitMeldungNachRumungUnfall").asTimeValue().setMillis(0);
        
    return data;
  }
  
  
  public void setzeVerkehrstaerkeMq(SystemObject mq, int vs)
  {
    DataModel dm = getConnection().getDataModel();
    AttributeGroup atg = dm.getAttributeGroup("atg.verkehrsDatenKurzZeitMq");
    Data data = getConnection().createData(atg);
    
    String[] werte = new String[] { "QKfz", "VKfz", "QLkw", "VLkw", "QPkw", "VPkw",
        "B", "BMax", "SKfz", "VgKfz", "ALkw", "KKfz", "KLkw", "KPkw", "QB", "KB", "VDelta" };

    for (String wert: werte)
    {
      if (wert.equals("QKfz"))
      {
        initialisiereVsDaten(data.getItem(wert), vs);
      }
      else
      {
        initialisiereVsDaten(data.getItem(wert), -1);
      }
    }
    
    Aspect asp = dm.getAspect("asp.analyse");
    DataDescription dd = new DataDescription(atg, asp);
    
    new Quelle().send(new ResultData(mq, dd, getConnection().getTime(), data));
  }
  
  public void setzeVerkehrstaerkeFs(SystemObject fs, int vs)
  {
    DataModel dm = getConnection().getDataModel();
    AttributeGroup atg = dm.getAttributeGroup("atg.verkehrsDatenKurzZeitFs");
    Data data = getConnection().createData(atg);
    
    String[] werte = new String[] { "T", "qKfz", "vKfz", "qLkw", "vLkw", "qPkw", "vPkw",
        "b", "sKfz", "vgKfz", "aLkw", "kKfz", "kLkw", "kPkw", "qB", "kB" };

    for (String wert: werte)
    {
      if (wert.equals("qKfz"))
      {
        initialisiereVsDaten(data.getItem(wert), vs);
      }
      else
      {
        if (wert.equals("T"))
        {
          data.getItem(wert).asTimeValue().setMillis(0); 
        }
        else
        {  
          initialisiereVsDaten(data.getItem(wert), -1);
        }  
      }
    }
    
    Aspect asp = dm.getAspect("asp.analyse");
    DataDescription dd = new DataDescription(atg, asp);
    
    new Quelle().send(new ResultData(fs, dd, getConnection().getTime(), data));
  }
  
  private void initialisiereVsDaten(Data data, int vs)
  {
    data.getItem("Wert").asUnscaledValue().set(vs);
    
    Data dataStatus = data.getItem("Status");
    Data dataErfassung = dataStatus.getItem("Erfassung");
    dataErfassung.getItem("NichtErfasst").asTextValue().setText("Nein");
    Data dataPIFormat = dataStatus.getItem("PlFormal");
    dataPIFormat.getItem("WertMin").asTextValue().setText("Nein");
    dataPIFormat.getItem("WertMax").asTextValue().setText("Nein");
    Data dataPILogisch = dataStatus.getItem("PlLogisch");
    dataPILogisch.getItem("WertMinLogisch").asTextValue().setText("Nein");
    dataPILogisch.getItem("WertMaxLogisch").asTextValue().setText("Nein");
    Data dataMessWertErsetzung = dataStatus.getItem("MessWertErsetzung");
    dataMessWertErsetzung.getItem("Implausibel").asTextValue().setText("Nein");
    dataMessWertErsetzung.getItem("Interpoliert").asTextValue().setText("Nein");
    
    Data dataGuete = data.getItem("Gte");
    dataGuete.getItem("Index").asScaledValue().set(1.0);
    dataGuete.getItem("Verfahren").asTextValue().setText("Standard");
  }
  
  class Quelle implements ClientSenderInterface
  {
    public final static long TIMEOUT = 1000;
    
    private boolean _warten = false;
    
    public void dataRequest(SystemObject object, DataDescription dataDescription, byte state)
    {
     
      if (_warten && state == ClientSenderInterface.START_SENDING)
      {
        synchronized(this)
        {
          notify();
        }
      }
    }

    public boolean isRequestSupported(SystemObject so, DataDescription dd)
    {
      return true;
    }
    
    public void send(ResultData resultData)
    {
      boolean result = true;
      try
      {
        getConnection().sendData(resultData);
      }
      catch (DataNotSubscribedException e)
      {
        try
        {
          getConnection().subscribeSender(this, resultData.getObject(), 
              resultData.getDataDescription(),  SenderRole.source());
        }
        catch (OneSubscriptionPerSendData e1)
        {
          // Ist schon subscribed?
          // tue nichts
        }
        try
        {
          getConnection().sendData(resultData);
        }
        catch (DataNotSubscribedException e1)
        {
          e1.printStackTrace();
          throw(e1);
        }
        catch (SendSubscriptionNotConfirmed e1)
        {
          try
          {
            synchronized(this)
            {
              _warten = true;
              wait(TIMEOUT);
              _warten = false;
            }
            getConnection().sendData(resultData);
          }
          catch(InterruptedException e2)
          {
            throw new RuntimeException("Warten auf Sendesteuerung unerwartet unterbrochen", e2);
          }
          catch(SendSubscriptionNotConfirmed e2)
          {
            throw new RuntimeException("Keine Sendesteuerung", e2);
          }
        }
      }
      catch (SendSubscriptionNotConfirmed e)
      {
        try
        {
          synchronized(this)
          {
            _warten = true;
            wait(TIMEOUT);
            _warten = false;
          }
          getConnection().sendData(resultData);
        }
        catch(InterruptedException e2)
        {
          throw new RuntimeException("Warten auf Sendesteuerung unerwartet unterbrochen", e2);
        }
        catch(SendSubscriptionNotConfirmed e2)
        {
          throw new RuntimeException("Keine Sendesteuerung", e2);
        }
      }
    }
    
  }
  
}
   

